import yaml
import time


DEFAULT_SWARM_MANAGEMENT_YAML_FILE = 'swarm-management.yml'


def GetInfoMsg():
    infoMsg = "One or more yaml files are used to configure the swarm.\r\n"
    infoMsg += "The yaml file 'swarm-management.yml' is used by default if no other files are specified.\r\n"
    infoMsg += "A yaml file may be specified by adding '-file' or '-f' to the arguments.\r\n"
    infoMsg += "Example: -f swarm-management-stacks.yml -f swarm-management-networks.yml\r\n"
    infoMsg += "Environment variables may be set with environment files.\r\n"
    infoMsg += "Multiple environment files may be set set with the 'env_files' property in the yaml file.\r\n"
    infoMsg += "Example: \r\n"
    infoMsg += "env_files: \r\n"
    infoMsg += "\t - 'environment.env'\r\n"
    infoMsg += "A preceding listed file will override any matching variables from the file listed above.\r\n"
    infoMsg += "Environment files may also be set set with the -e/-env argument.\r\n"
    infoMsg += "Setting the -e/-env argument will override matching environment variables in the files from the 'env_files' yaml property.\r\n"
    infoMsg += "Example: -e environment.env\r\n"
    return infoMsg


def GetYamlString(yamlFile):
    with open(yamlFile) as f:
        yamlString = f.read() + "\r\n"
    return yamlString


def GetYamlData(yamlFiles):
    yamlStrings = ""
    for yamlFile in yamlFiles:
        yamlStrings += GetYamlString(yamlFile)
    yamlData = yaml.load(yamlStrings)
    if yamlData == None:
        errorMsg = "No .yml files where discovered!\r\n"
        errorMsg += GetInfoMsg()
        raise Exception(errorMsg)
    return yamlData


def GetArgumentValues(arguments, argumentType, ignoreArgumentsWithPrefix = "-"):
    argumentValues = []
    for i in range(len(arguments)):
        currentArgumentType = arguments[i]
        if currentArgumentType == argumentType:
            for j in range(len(arguments)):
                argumentValue = arguments[i]
                if not(argumentValue.startswith(ignoreArgumentsWithPrefix)):
                    argumentValues.append(argumentValue)
    return argumentValues


def LoadYamlDataFromFiles(arguments, defaultYamlFiles=[DEFAULT_SWARM_MANAGEMENT_YAML_FILE]):
    yamlFiles = GetArgumentValues(arguments, '-file')
    yamlFiles += GetArgumentValues(arguments, '-f')
    if len(yamlFiles) == 0:
        yamlFiles = defaultYamlFiles
    yamlData = GetYamlData(yamlFiles)
    return yamlData


def GetEnvironmnetVariablesFiles(arguments, yamlData):
    envFiles = []
    if 'env_files' in yamlData:
        envFiles += yamlData['env_files']
    envFiles += GetArgumentValues(arguments, '-env')
    envFiles += GetArgumentValues(arguments, '-e')
    return envFiles


def GetProperties(arguments, propertyType, errorInfoMsg, yamlData):
    properties = {}
    if propertyType in yamlData:
        properties = yamlData[propertyType]
    return properties


def TimeoutCounter(secTimeout):
    startTime = time.time()
    elapsedTime = time.time() - startTime
    timeLeft = secTimeout - int(elapsedTime)
    printedTime = timeLeft
    while elapsedTime < secTimeout:
        timeLeft = secTimeout - int(elapsedTime)
        if timeLeft < printedTime:
            printedTime = timeLeft
            print("Restarting Swarm in %d seconds" % printedTime)
        elapsedTime = time.time() - startTime
